# -*- coding: utf-8 -*-
import os

from robot.libraries.BuiltIn import BuiltIn
from PIL import Image, ImageDraw


class RobotS2LScreenshot:
  '''
  Provides additional keywords to augment screenshots capabilities of 
  RobotFramework's Selenium2Library, namely cropping and masking to help with 
  perceptual diff testing.
  '''

  def capture_and_crop_page_screenshot(self, filename, locator=None):
    '''
    Captures page screenshot and crops to specified element.
    '''
    s2l = BuiltIn().get_library_instance('Selenium2Library')
    # note: filename is required because otherwise we don't have reference to 
    #       auto-generated filename
    s2l.capture_page_screenshot(filename)

    element = s2l._element_find(locator, True, False)
    if element is None:
      raise AssertionError("Could not locate element for '%s'" % (locator))
    x, y = element.location['x'], element.location['y']
    w, h = element.size['width'], element.size['height']

    self.crop_image(filename, [x, y, x+w, y+h])


  def capture_and_mask_page_screenshot(self, filename, locators=None,
                                       rects=None):
    '''
    Captures page screenshot and masks specified element.
    '''
    s2l = BuiltIn().get_library_instance('Selenium2Library')
    # note: filename is required because otherwise we don't have reference to 
    #       auto-generated filename
    s2l.capture_page_screenshot(filename)

    rectangles = []

    for locator in locators or []:
      element = s2l._element_find(locator, True, False)
      if element is None:
        raise AssertionError("Could not locate element for '%s'" % (locator))
      x, y = element.location['x'], element.location['y']
      w, h = element.size['width'], element.size['height']
      rectangles.append([x, y, x+w, y+h])

    for rect in rects or []:
      try:
        x, y, w, h = [int(i) for i in rect.split(',')]
        rectangles.append([x, y, x+w, y+h])
      except:
        raise AssertionError("Could not locate rectangle for '%s'" % (rect))

    self.mask_image(filename, rectangles)


  def crop_image(self, screenshot, rectangle):
    '''
    Crops image to defined rectangle (in [x0, y0, x1, y1] format).
    '''
    variables = BuiltIn().get_variables()
    outputdir = variables['${OUTPUTDIR}']
    screenshot = os.path.join(outputdir, screenshot)

    img = Image.open(screenshot)
    area = img.crop(tuple(rectangle))
    area.save(screenshot, 'png')


  def mask_image(self, screenshot, rectangles):
    '''
    Masks areas defined by rectangles (in [x0, y0, x1, y1] format) with 
    transparency.
    '''
    variables = BuiltIn().get_variables()
    outputdir = variables['${OUTPUTDIR}']
    screenshot = os.path.join(outputdir, screenshot)

    img = Image.open(screenshot)
    draw = ImageDraw.Draw(img)
    for rectangle in rectangles:
      # "delete" specified rectangle area
      draw.rectangle(rectangle, fill=(255,255,255,0))
    img.save(screenshot, 'PNG')



# -*- coding: utf-8 -*-
'''
Nose wrapper that runs the test files using robot and then compare the 
screenshots using perceptualdiff.
'''
import glob
import os
import subprocess
import string
import sys
import unittest

from robot.running.builder import TestSuiteBuilder


class TestRobotS2LScreenshot(unittest.TestCase):
  def robot_launcher(self, test_file, test_case):
    # run robot test
    p = subprocess.Popen(['pybot', '-t', test_case, test_file],
                         close_fds=True,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    sys.stdout.write(p.communicate()[0])
    self.assertEqual(
      p.returncode, 0, 
      msg='Robot test case failed to complete successfully: %r' % p.returncode)

    # diff screenshots with baseline
    curr_img = test_case + '.png'
    base_img = 'baseline-' + curr_img
    diff_img = 'diff-' + curr_img
    d = subprocess.Popen(['perceptualdiff', base_img, curr_img, 
                          '-output', diff_img],
                         close_fds=True,
                         stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    sys.stdout.write(d.communicate()[0])
    self.assertEqual(
      d.returncode, 0, 
      msg='Screenshots are different: %r' % d.returncode)
                          
                          

robot_files = glob.glob(os.path.join(os.path.dirname(__file__), 'test*.txt'))
for robot_file in robot_files:
  robot_suite = TestSuiteBuilder().build(robot_file)
  for robot_case in robot_suite.tests:
    def maker(tf, tc):
      def test_run(self):
        self.robot_launcher(tf, tc)
      return test_run
    test_name = string.replace(robot_suite.name + '.' + robot_case.name, ' ', '_')
    setattr(TestRobotS2LScreenshot, test_name, maker(robot_file, robot_case.name))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
  README = f.read()

requires = [
  'robotframework >= 2.8.7',
  'robotframework-selenium2library >= 1.6.0',
  'selenium >= 2.45.0',
  'Pillow >= 2.2.1',
  ]

test_requires = [
  'nose >= 1.3.0',
  ]

classifiers = [
  'Development Status :: 4 - Beta',
  'Environment :: Web Environment',
  'Intended Audience :: Developers',
  'License :: OSI Approved :: MIT License',
  'Natural Language :: English',
  'Operating System :: OS Independent',
  'Programming Language :: Python',
  'Topic :: Software Development :: Quality Assurance',
  'Topic :: Software Development :: Testing',
]

setup(
  name='RobotS2LScreenshot',
  version='0.1.1',
  description="Provides additional keywords to augment screenshots capabilities of RobotFramework's Selenium2Library, namely cropping and masking to help with perceptual diff testing.",
  long_description=README,
  classifiers=classifiers,
  platforms=['any'],
  url='https://github.com/canaryhealth/RobotS2LScreenshot',
  license='MIT',
  packages=find_packages(),
  include_package_data=True,
  zip_safe=True,
  test_suite='test',
  install_requires=requires,
  tests_require=test_requires,
)
